@@ -2,12 +2,17 @@
  *                 Mathematical Object Library                 *
  *          class function: declaration for functions          *
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 2.4.0                   *
+ *                   GNU/linux version 2.6.0                   *
  *            software under General Public License            *
  ***************************************************************
- * copyright © 2005,2006,2008,2009 COLLARD Christophe
+ * copyright © 2005,2006,2008,2009,2011 COLLARD Christophe
+ * copyright © 2005,2006,2008,2009,2011 Centre National de la Recherche Scientifique
+ * copyright © 2005,2006,2008,2009,2011 Arts et Métiers ParisTech
+ * copyright © 2005,2006 Université de Valenciennes et du Hainaut Cambrésis
  * copyright © 2005,2006,2008,2009 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
  * copyright © 2005,2006 Laboratoire de Mathématiques et ses Applications de Valenciennes (LAMAV)
+ * copyright © 2011 Laboratoire d'Etude des Microstructures et de Mécanique des Matériaux (LEM3 - CNRS)
+ * copyright © 2011 Centre d'Elaboration de Matériaux et d'Etudes Structurales (CEMES - CNRS)
  ***************************************************************/
 
 /*! \namespace mol
@@ -40,11 +45,16 @@
     </FONT>
     \endhtmlonly
 
-    \author copyright \htmlonly &#169; \endhtmlonly 2005, 2006, 2008, 2009 Christophe COLLARD \n
+    \author copyright \htmlonly &#169; \endhtmlonly 2005, 2006, 2008, 2009, 2011 Christophe COLLARD \n
+            copyright \htmlonly &#169; 2005, 2006, 2008, 2009, 2011 Centre National de la Recherche Scientifique \endhtmlonly \n
+	    copyright \htmlonly &#169; 2005, 2006, 2008, 2009, 2011 Arts et M&#233;tiers ParisTech \endhtmlonly \n
+	    copyright \htmlonly &#169; 2005, 2006 Universit&#233; de Valenciennes et du Hainaut Cambr&#233;sis \endhtmlonly \n
 	    copyright \htmlonly &#169; 2005, 2006, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	    copyright \htmlonly &#169; 2005, 2006 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-    \version 2.4.0
-    \date 2005-2010
+	    copyright \htmlonly &#169; 2011 Laboratoire d'Etude des Microstructures et de M&#233;canique des Mat&#233;riaux (LEM3 - CNRS) \endhtmlonly \n
+            copyright \htmlonly &#169; 2011 Centre d'Elaboration de Mat&#233;riaux et d'Etudes Structurales (CEMES - CNRS) \endhtmlonly \n
+    \version 2.6.0
+    \date 2005-2011
     \bug none
     \warning under construction (an interpreter will be available later)
 */
@@ -56,6 +66,18 @@
 #if !defined(_FUNCTIONS_H)
 #define _FUNCTIONS_H
 
+#if !defined (__STDLIB_H)
+#include <stdlib.h>
+#endif
+
+#if !defined (__IOSTREAM_H)
+#include <iostream>
+#endif
+
+#if !defined(__VECTORS_H)
+#include "vectors.h"
+#endif
+
 #if !defined(__VECTORS_H)
 #include "vectors.h"
 #endif
