@@ -2,15 +2,20 @@
  *                 Mathematical Object Library                 *
  *       class spmatrix : declarations for sparse matrix       *
  *                    simula+@metz.ensam.fr                    *
- *		     GNU/linux version 2.4.0                   *
+ *		     GNU/linux version 2.6.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 2004,2005 LAGOUCHE Sophie
  * copyright © 2004,2005 CREUSE Emmanuel
  * copyright © 2006 FACHE Damien
- * copyright © 2006,2007,2008,2009 COLLARD Christophe
+ * copyright © 2006,2007,2008,2009,2011 COLLARD Christophe
+ * copyright © 2004,2005,2006,2007,2008,2009,2011 Centre National de la Recherche Scientifique
+ * copyright © 2004,2005,2006,2007,2008,2009,2011 Arts et Métiers ParisTech
+ * copyright © 2004,2005,2006,2007 Université de Valenciennes et du Hainaut Cambrésis
  * copyright © 2004,2005,2006,2007,2008,2009 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
  * copyright © 2004,2005,2006,2007 Laboratoire de Mathématiques et ses Applications de Valenciennes (LAMAV)
+ * copyright © 2011 Laboratoire d'Etude des Microstructures et de Mécanique des Matériaux (LEM3 - CNRS)
+ * copyright © 2011 Centre d'Elaboration de Matériaux et d'Etudes Structurales (CEMES - CNRS)
  ***************************************************************/
 
 /*! \namespace mol
@@ -108,11 +113,16 @@
   \authors copyright \htmlonly &#169; \endhtmlonly 2004, 2005 Sophie LAGOUCHE \n
            copyright \htmlonly &#169; \endhtmlonly 2004, 2005 Emmanuel CREUSE \n
            copyright \htmlonly &#169; \endhtmlonly 2006 Damien FACHE \n
-           copyright \htmlonly &#169; \endhtmlonly 2006, 2007, 2008, 2009 Christophe COLLARD \n
+           copyright \htmlonly &#169; \endhtmlonly 2006, 2007, 2008, 2009, 2011 Christophe COLLARD \n
+	   copyright \htmlonly &#169; 2004, 2005, 2006, 2007, 2008, 2009, 2011 Centre National de la Recherche Scientifique \endhtmlonly \n
+	   copyright \htmlonly &#169; 2004, 2005, 2006, 2007, 2008, 2009, 2011 Arts et M&#233;tiers ParisTech \endhtmlonly \n
+	     copyright \htmlonly &#169; 2004, 2005, 2006, 2007 Universit&#233; de Valenciennes et du Hainaut Cambr&#233;sis \endhtmlonly \n
 	   copyright \htmlonly &#169; 2004, 2005, 2006, 2007, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	   copyright \htmlonly &#169; 2004, 2005, 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-  \version 2.4.0
-  \date 2004-2010
+	   copyright \htmlonly &#169; 2011 Laboratoire d'Etude des Microstructures et de M&#233;canique des Mat&#233;riaux (LEM3 - CNRS) \endhtmlonly \n
+	   copyright \htmlonly &#169; 2011 Centre d'Elaboration de Mat&#233;riaux et d'Etudes Structurales (CEMES - CNRS) \endhtmlonly \n
+  \version 2.6.0
+  \date 2004-2011
   \bug multiple allocation does not work with g++ compiler (ex. A(2,4) = A(3,3) = A(5,5) = 2;)
   \warning none
 */
@@ -125,6 +135,10 @@
 #define _SP_MATRIX
 
 
+#if !defined (__STDLIB_H)
+#include <stdlib.h>
+#endif
+
 #if !defined(__IOSTREAM_H)
 #include <iostream>
 #endif
