@@ -2,13 +2,17 @@
  *                 Mathematical Object Library                 *
  *     class symmatrix:  declarations for symmetric matrix     *
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 2.4.0                   *
+ *                   GNU/linux version 2.6.0                   *
  *            software under General Public License            *
  ***************************************************************
- * copyright © 2002,2003,2004,2005,2006,2007,2008,2009 COLLARD Christophe
+ * copyright © 2002,2003,2004,2005,2006,2007,2008,2009,2010 COLLARD Christophe
  * copyright © 2002,2003,2004 CREUSE Emmanuel
- * copyright © 2002,2003,2004,2005,2006,2007,2008,2009 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
+ * copyright © 2002,2003,2004,2005,2006,2007,2008,2009,2010 Centre National de la Recherche Scientifique
+ * copyright © 2002,2003,2004,2005,2006,2007,2008,2009,2010 Arts et Métiers ParisTech
+ * copyright © 2002,2003,2004,2005,2006,2007 Université de Valenciennes et du Hainaut Cambrésis
+ * copyright © 2002,2003,2004,2005,2006,2007,2008,2009,2010 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
  * copyright © 2002,2003,2004,2005,2006,2007 Laboratoire de Mathématiques et ses Applications de Valenciennes (LAMAV)
+ * copyright © 2010 Centre d'Elaboration de Matériaux et d'Etudes Structurales (CEMES - CNRS)
  ***************************************************************/
 
 /*! \namespace mol
@@ -42,12 +46,16 @@
     \endhtmlonly
 
 
-    \authors copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Christophe COLLARD \n
+    \authors copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Christophe COLLARD \n
              copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004 Emmanuel CREUSE \n
-	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
+             copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Centre National de la Recherche Scientifique \endhtmlonly \n
+	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Arts et M&#233;tiers ParisTech \endhtmlonly \n
+	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007 Universit&#233; de Valenciennes et du Hainaut Cambr&#233;sis \endhtmlonly \n
+	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-    \version 2.4.0
-    \date 2002-2010
+	     copyright \htmlonly &#169; 2010 Centre d'Elaboration de Mat&#233;riaux et d'Etudes Structurales (CEMES - CNRS) \endhtmlonly \n
+    \version 2.6.0
+    \date 2002-2011
     \bug none
     \warning none
 */
@@ -131,6 +139,7 @@ template <class T> class symmatrix : public virtual matrix<T>
     friend symmatrix<T> t (const symmatrix<T>& mat) {return mat;}        // optimizes tranposition operator for symmatrix
     template <class Tf> friend symmatrix<Tf> gauss (const symmatrix<Tf>&);  // computes symmatrix inversion with gauss algorithm
     template <class Tf> friend symmatrix<Tf> abs (const symmatrix<Tf>&);    // returns the symmatrix absolute value
+    T p_norm (int); // computes p-norm for matrix
     template <class Tf> friend symmatrix<Tf> Id2s (int);                    // returns identity matrix
 };
 
@@ -204,7 +213,7 @@ template <class T> symmatrix<T>::symmatrix (const matrix<T>& mat)
   for (int i=1; i<=nbRows; i++)
     { mrows[i-1] = i;
       for (int j=1; j<=i; j++)
- 	{ assert (abs(mat(i,j)-mat(j,i))<epsilon); // needed by symmatrix = matrix
+ 	{ assert (relative_error (mat(i,j), mat(j,i))); // needed by symmatrix = matrix
 	  (*this)(i,j) = mat(i,j);
 	}
     }
@@ -268,7 +277,7 @@ template <class T> symmatrix<T>& symmatrix<T>::operator = (const matrix<T>& mat)
 
   for (int i=1; i<=nbRows; i++)
     for (int j=1; j<=i; j++)
-      { assert (abs(mat(i,j)-mat(j,i))<epsilon); // needed by symmatrix = matrix
+      { assert (relative_error (mat(i,j), mat(j,i))); // needed by symmatrix = matrix
         (*this)(i,j) = mat(i,j);
       }
 
@@ -441,7 +450,7 @@ template <class T> symmatrix<T>& symmatrix<T>::operator += (const matrix<T>& mat
 
   for (int i=1; i<=nbRows; i++)
     for (int j=1; j<=i; j++)
-      { assert(abs(mat(i,j)-mat(j,i))<epsilon);
+      { assert(relative_error (mat(i,j), mat(j,i)));
         (*this)(i,j) += mat(i,j);
       }
 
@@ -472,7 +481,7 @@ template <class T> symmatrix<T>& symmatrix<T>::operator -= (const matrix<T>& mat
 
   for (int i=1; i<=nbRows; i++)
     for (int j=i; j<=nbColumns; j++)
-      { assert(abs(mat(i,j)-mat(j,i))<epsilon);
+      { assert (relative_error (mat(i,j), mat(j,i)));
         (*this)(i,j) -= mat(i,j);
       }
 
@@ -527,6 +536,35 @@ template <class Tf> symmatrix<Tf> abs (const symmatrix<Tf>& mat)
 }
 
 
+/*!
+  \brief Computes the p-norm for symmetric matrix.
+
+  Let \f$ A \f$ be a \f$ n \times n \f$ symmetric matrix. Then, its p-norm is given by: \n
+  \f$ \left\| A \right\|_p = \displaystyle \left( \sum_{i=1}^{n} \sum_{j=1}^{n} \left| a_{ij} \right|^p \right)^{1/p} \f$.\n
+  The special case p = 2 is the Frobenius norm also called the Hilbert-Schmidt norm.
+
+  \param p value for p-norm
+  \return \f$ \left\| A \right\|_p \f$
+
+*/
+
+//-----------------------------------------------
+template <class T> T symmatrix<T>::p_norm (int p)
+//-----------------------------------------------
+{
+  assert (p > 0);
+
+  T sum = 0;
+  for (int i=1; i<=nbRows; i++)
+    { sum += pow (mrows[i-1][i], p);
+      for (int j=1; j<i; j++)
+	sum += 2 * pow (mrows[i-1][j], p);
+    }
+
+  return pow (sum, (long double) 1/p);
+}
+
+
 //--------------------------------------------
 template <class Tf> symmatrix<Tf> Id2s (int n)
 //--------------------------------------------
