@@ -2,11 +2,15 @@
  *                 Mathematical Object Library                 *
  *class symtensor4: declaration for symmetric 4th order tensors*
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 2.4.0                   *
+ *	             GNU/linux version 2.6.0                   *
  *            software under General Public License            *
  ***************************************************************
- * copyright © 2009,2010 COLLARD Christophe
+ * copyright © 2009,2010,2011 COLLARD Christophe
+ * copyright © 2009,2010,2011 Centre National de la Recherche Scientifique
+ * copyright © 2009,2010,2011 Arts et Métiers ParisTech
  * copyright © 2009,2010 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
+ * copyright © 2011 Laboratoire d'Etude des Microstructures et de Mécanique des Matériaux (LEM3 - CNRS)
+ * copyright © 2010,2011 Centre d'Elaboration de Matériaux et d'Etudes Structurales (CEMES - CNRS)
  ***************************************************************/
 
 /*! \namespace mol
@@ -42,10 +46,14 @@
     When fourth order tensors have the following symmetry properties \f$ T_{ijkl} = T_{jikl} = T_{ijlk} \f$, the storage and the operations on these tensors can be improved by using a matrix storage instead of the one given by the tensor4 class (sets of matrices). To do so, we associate with the tensor \f$ \boldsymbol{T} \f$ in \f$ \Bbb{R}^n \times \Bbb{R}^n \times \Bbb{R}^m \times \Bbb{R}^m \f$ the matrix \f$ \boldsymbol{M} \f$ in \f$ \Bbb{R}^{2n} \times \Bbb{R}^{2m} \f$ defined by: \n
 
 
-    \authors copyright \htmlonly &#169; \endhtmlonly 2009, 2010 Christophe COLLARD \n
+    \authors copyright \htmlonly &#169; \endhtmlonly 2009, 2010, 2011 Christophe COLLARD \n
+             copyright \htmlonly &#169; 2009, 2010, 2011 Centre National de la Recherche Scientifique \endhtmlonly \n
+	     copyright \htmlonly &#169; 2009, 2010, 2011 Arts et M&#233;tiers ParisTech \endhtmlonly \n
 	     copyright \htmlonly &#169; 2009, 2010 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
-    \version 2.4.0
-    \date 2009-2010
+	     copyright \htmlonly &#169; 2011 Laboratoire d'Etude des Microstructures et de M&#233;canique des Mat&#233;riaux (LEM3 - CNRS) \endhtmlonly \n
+	     copyright \htmlonly &#169; 2010, 2011 Centre d'Elaboration de Mat&#233;riaux et d'Etudes Structurales (CEMES - CNRS) \endhtmlonly \n
+    \version 2.6.0
+    \date 2009-2011
     \bug none
     \warning none
 */
@@ -78,6 +86,10 @@
 #include "parameters.h"
 #endif
 
+#if !defined(__COLORS_H)
+#include "colors.h"
+#endif
+
 #if !defined(__MATRIX_H)
 #include "MOL++/matrix.h"
 #endif
@@ -112,23 +124,24 @@ namespace mol
 template <class T> class symtensor4
 //=================================
 {
-  private:
-    matrix<T> mat;
+  protected:
+    matrix<T> mat; // matrix stored with abaqus convention
 
   public:
     symtensor4 () {}
     symtensor4 (int, bool=true, T=0);
     symtensor4 (int, int, int, int, bool=true, T=0);
 
-    symtensor4 (const matrix<T>& mt) {mat = mt;}       // cast constructor (converts matrix    -> symtensor4)
-    symtensor4 (const symmatrix<T>& smt) {mat = smt;}  // cast constructor (converts symmatrix -> symtensor4)
+    symtensor4 (const matrix<T>&);      // cast constructor (converts matrix    -> symtensor4)
+    symtensor4 (const symmatrix<T>&);   // cast constructor (converts symmatrix -> symtensor4)
 
-    symtensor4 (symtensor4<T>* st) {mat = &(st->mat);}   // copy constructor for temporary objects
-    symtensor4 (matrix<T>* st) {mat = st;}               // copy constructor for temporary objects
-    symtensor4 (symmatrix<T>* smat) {mat = smat;}        // copy constructor for temporary objects
+    symtensor4 (symtensor4<T>* st) {mat = &(st->mat); (*st).mat&=matrix<T>();}  // copy constructor for temporary objects
 
     ~symtensor4 () {}       // destructor
 
+    operator tensor4<T> () const; // cast operator : symtensor4 -> tensor4
+    operator matrix<T> () const;  // cast operator : symtensor4 -> matrix
+
     virtual int dim1 () const {return .5 * (sqrt(1.+8*mat.Rows()) - 1);}     // returns the tensor 1st component size : i
     virtual int dim2 () const {return .5 * (sqrt(1.+8*mat.Rows()) - 1);}     // returns the tensor 2nd component size : j
     virtual int dim3 () const {return .5 * (sqrt(1.+8*mat.Columns()) - 1);}  // returns the tensor 3rd component size : k
@@ -141,8 +154,8 @@ template <class T> class symtensor4
     virtual T& operator () (int, int, int, int) const;     // returns the element Pijkl : P(tensor3,tensor2,line,column)
     virtual vector<T>& operator [] (int i) const {return mat[i];}
     //    virtual symtensor4<T>& operator = (symtensor4<T>* st) {mat =& st->mat;}  // allocates the data from the right hand temporary object to the left hand object - no copy (deletes temporary object)
-    virtual symtensor4<T>& operator &= (const symtensor4<T>& st) {mat &= st.mat;}  // copies symtensor4 without size check (use this operator with great care)
-    virtual symtensor4<T>& operator &= (symtensor4<T>* st) {mat &=& (st->mat);}    // allocates the data from the right hand temporary object to the left hand object (deletes  temporary object)
+    virtual symtensor4<T>& operator &= (const symtensor4<T>& st) {mat &= st.mat; return (*this);}  // copies symtensor4 without size check (use this operator with great care)
+    virtual symtensor4<T>& operator &= (symtensor4<T>* st) {mat &=& (st->mat);   return (*this);}  // allocates the data from the right hand temporary object to the left hand object (deletes  temporary object)
     template <class Tf> friend symtensor4<Tf> operator +  (const symtensor4<Tf>&, const symtensor4<Tf>&);
     template <class Tf> friend symtensor4<Tf> operator -  (const symtensor4<Tf>&, const symtensor4<Tf>&);
     template <class Tf> friend Tf             operator |  (const symtensor4<Tf>&, const symtensor4<Tf>&); // T_ijkl  R_ijkl
@@ -188,7 +201,7 @@ template <class T> class symtensor4
     //----------------------------------
     // operations with 4th order tensors
     //----------------------------------
-    symtensor4 (const tensor4<T>& tsr) { mat = abaqus::symtensor2matrix<T> (tsr);}  // cast constructor (converts tensor4   -> symtensor4)
+    symtensor4 (const tensor4<T>& tsr) {mat = abaqus::symtensor2matrix<T> (tsr);}  // cast constructor (converts tensor4   -> symtensor4)
     symtensor4 (tensor4<T>* tsr) {mat = &(abaqus::symtensor2matrix<T> (*tsr));}     // copy constructor for temporary objects
 
     virtual symtensor4<T>& operator &= (const tensor4<T>& tsr);  // copies symtensor4 without size check (use this operator with great care)
@@ -209,18 +222,19 @@ template <class T> class symtensor4
     template <class Tf> friend tensor4<Tf> operator || (const tensor4<Tf>& t4, const symtensor4<Tf>& st) {return t4 || abaqus::matrix2symtensor (st.mat);}
 
     template <class Tf> friend symtensor4<Tf> Id4sym (int); // returns identity tensor
-    //    template <class Tf> friend tensor4<Tf> change_basis (const matrix<Tf>&, const tensor4<Tf>&);
+    template <class Tf> friend symtensor4<Tf> change_basis (const matrix<Tf>&, const symtensor4<Tf>&);
+    symtensor4<T> inv ();  // computes inversion operator
+    template <class Tf> friend symtensor4<Tf> gauss (symtensor4<Tf>& tsr) {return tsr.inv();}  // computes matrix inversion with gauss algorithm
+    T p_norm (int p);      // computes p-norm for matrix
+
     template <class Tf> friend ostream& operator << (ostream&, const symtensor4<Tf>&);
     template <class Tf> friend istream& operator >> (istream&, const symtensor4<Tf>&);
-    virtual symtensor4<T> inv () {return mat.inv();}  // computes inversion operator (do not use this one since the algorithm is too slow)
-    template <class Tf> friend symtensor4<Tf> gauss (symtensor4<Tf> tsr) {return gauss(tsr.mat);}  // computes matrix inversion with gauss algorithm
-    template <class Tf> friend bool inv (symtensor4<Tf>& tsr, long double eps=epsilon) {return mol::inv (tsr.mat, eps);}  // same function as gauss but returns the inversion result 0/1
 
     // i/o disk
     virtual void save (const char*);   // writes to disk
     virtual void save (const string&); // writes to disk
     virtual void read (const char*);   // reads from disk
-    virtual void read (const string&);   // reads from disk
+    virtual void read (const string&); // reads from disk
 
     symtensor4<T>& approximation ();
 };
@@ -272,6 +286,125 @@ template <class T> symtensor4<T>::symtensor4 (int i, int j, int k, int l, bool i
 }
 
 
+/*!
+  \brief Cast constructor
+
+  Converts and stores the matrix 'real_mat' given in mol spacename into the matrix of symmetric \f$ 4^\text{th} \f$ order tensor given in abaqus spacename
+
+  \param true_mat matrix defined in mol spacename
+*/
+
+//----------------------------------------------------------------------
+template <class T> symtensor4<T>::symtensor4 (const matrix<T>& true_mat)
+//----------------------------------------------------------------------
+{
+  assert (true_mat.Rows() && true_mat.Columns());
+
+  mat = true_mat;
+  assert (.5 * (sqrt(1.+8*mat.Rows()) - 1) == dim1());     // checks matrix size is valid
+  assert (.5 * (sqrt(1.+8*mat.Columns()) - 1) == dim3());  // checks matrix size is valid
+
+
+  for (int i=1; i<=dim1(); i++)
+    for (int j=dim3()+1; j<=mat.Columns(); j++)
+      mat(i,j) *= 0.5 * sqrt(2);
+
+  for (int i=dim1()+1; i<=mat.Rows(); i++)
+    { for (int j=1; j<=dim3(); j++)
+	mat(i,j) *= 0.5 * sqrt(2);
+      for (int j=dim3()+1; j<=mat.Columns(); j++)
+	mat(i,j) *= 0.5;
+    }
+}
+
+
+/*!
+  \brief Cast constructor
+
+  Converts and stores the symmetric matrix 'real_smat' given in mol spacename into the matrix of symmetric \f$ 4^\text{th} \f$ order tensor given in abaqus spacename
+
+  \param true_smat matrix defined in mol spacename
+*/
+
+//--------------------------------------------------------------------------
+template <class T> symtensor4<T>::symtensor4 (const symmatrix<T>& true_smat)
+//--------------------------------------------------------------------------
+{
+  assert (true_smat.Rows() && true_smat.Columns());
+
+  mat = true_smat;
+  assert (.5 * (sqrt(1.+8*mat.Rows()) - 1) == dim1()); // checks symmatrix size is valid
+
+  for (int i=1; i<=dim1(); i++)
+    for (int j=dim3()+1; j<=mat.Columns(); j++)
+      mat(i,j) *= 0.5 * sqrt(2);
+
+  for (int i=dim1()+1; i<=mat.Rows(); i++)
+    { for (int j=1; j<=dim3(); j++)
+	mat(i,j) *= 0.5 * sqrt(2);
+      for (int j=dim3()+1; j<=mat.Columns(); j++)
+	mat(i,j) *= 0.5;
+    }
+}
+
+
+/*!
+  \brief Cast operator for symmetric \f$ \displaystyle 4^{th} \f$ order tensors.
+
+  Converts a symmetric \f$ \displaystyle 4^{th} \f$ order tensor into a \f$ \displaystyle 4^{th} \f$ order tensor.
+*/
+
+//------------------------------------------------------------
+template <class T> symtensor4<T>::operator tensor4<T> () const
+//------------------------------------------------------------
+{
+  int p = dim1();
+  int q = dim3();
+  assert (p && q);
+
+  tensor4<T> tsr (p, p, q, q, false);
+
+  for (int i=1; i<=p; i++)
+    for (int j=1; j<=p; j++)
+      for (int k=1; k<=q; k++)
+	for (int l=1; l<=q; l++)
+	  tsr (i,j,k,l) = (*this) (i,j,k,l);
+
+  return tsr;
+}
+
+
+/*!
+  \brief Cast operator for symmetric \f$ \displaystyle 4^{th} \f$ order tensors.
+
+  Converts a symmetric \f$ \displaystyle 4^{th} \f$ order tensor into a matrix.
+*/
+
+//-----------------------------------------------------------
+template <class T> symtensor4<T>::operator matrix<T> () const
+//-----------------------------------------------------------
+{
+  int p = dim1();
+  int q = dim3();
+  assert (p && q);
+
+  matrix<T> true_mat = mat;
+
+  for (int i=1; i<=dim1(); i++)
+    for (int j=dim3()+1; j<=mat.Columns(); j++)
+      true_mat(i,j) *= sqrt(2);
+
+  for (int i=dim1()+1; i<=mat.Rows(); i++)
+    { for (int j=1; j<=dim3(); j++)
+	true_mat(i,j) *= sqrt(2);
+      for (int j=dim3()+1; j<=mat.Columns(); j++)
+	true_mat(i,j) *= 2;
+    }
+
+  return true_mat;
+}
+
+
 /*!
   \brief Partial memory allocation
 
@@ -291,6 +424,7 @@ template <class T> void symtensor4<T>::create (int ntensor3)
   mat.create (p,p);
 }
 
+
 /*!
   \brief Full memory allocation without initialization of the tensor coordinates
 
@@ -656,13 +790,30 @@ template <class Tf> tensor3<Tf> operator * (const vector<Tf>& v, const symtensor
   assert (st.dim1() && st.dim3());
   assert (v.dim() == st.dim1());
 
-  tensor3<Tf> t3 (st.dim1(), st.dim3(), st.dim3());
+  int n = st.dim1();
+  int m = st.dim3();
+  tensor3<Tf> t3;
+  t3.create (n);
 
-  for (int j=1; j<=st.dim1(); j++)
-    for (int k=1; k<=st.dim3(); k++)
-      for (int l=1; l<=st.dim3(); l++)
-	for (int i=1; i<=st.dim1(); i++)
-	  t3[j][k][l] += st (i,j,k,l) * v[i];
+  for (int i=1,p=1; i<=n; i++)
+    { vector<Tf> sum = st.mat[i] * v[i];
+
+      for (int j=i+1; j<=n; j++, p++)
+	sum += st.mat[n+p] * v[j];
+
+      int q = n+i-1;
+      for (int j=1,k=0; j<i; j++,k+=n-j)
+	sum += st.mat[q+k] * v[j];
+
+      symmatrix<Tf> smat(m,false);
+      for (int k=1,q=1; k<=m; k++)
+	{ smat(k,k) = sum[k];
+	  for (int l=k+1; l<=m; l++, q++)
+	    smat(l,k) = sum[m+q];
+	}
+
+      t3[i] = smat;
+    }
 
   return t3;
 }
@@ -869,7 +1020,10 @@ template <class Tf> symtensor4<Tf> operator ^ (const symtensor2<Tf>& st1, const
 	v2[m+p] = st2(i,j);
     }
 
-  return (v1 ^ v2);
+  symtensor4<Tf> st;
+  st.mat = (v1 ^ v2);
+
+  return st;
 }
 
 
@@ -1014,6 +1168,106 @@ template <class Tf> symtensor4<Tf> Id4sym (int n)
 }
 
 
+/*!
+  \brief Computes the change of basis for a symmetric \f$ 4^{th} \f$ order tensor.
+
+  \f$ \displaystyle T^\prime_{pqmn} = P_{pi} P_{qj} P_{mk} P_{nl} T_{ijkl} \f$. \n
+
+  \param P change of basis matrix
+  \param tsr symmetric tensor T in the old basis
+
+  \return \f$ T^\prime \f$ symmetric tensor in the new basis
+*/
+
+//----------------------------------------------------------------------------------------------
+template <class Tf> symtensor4<Tf> change_basis (const matrix<Tf>& P, const symtensor4<Tf>& tsr)
+//----------------------------------------------------------------------------------------------
+{
+  int ni = P.Rows();
+  int nb = tsr.mat.Rows();
+  assert (ni == P.Columns());
+  assert (ni == tsr.dim1()  &&  ni == tsr.dim3());
+
+  symtensor4<Tf> Tsr;
+
+  matrix<Tf> L (tsr.mat.Rows(), tsr.mat.Columns());
+  for (int p=1, pq=ni; p<=ni; p++)
+    for (int q=p; q<=ni; q++)
+      { if (p!=q) pq++;
+	for (int i=1, ij=ni; i<=ni; i++)
+	  for (int j=i; j<=ni; j++)
+	    { if (i!=j) ij++;
+	      if (p==q && i==j) L(p,i) = P(p,i) * P(p,i);
+	      else if (p==q) L(p,ij) = 2 * P(p,i) * P(p,j);
+	           else if (i==j) L(pq,i) = P(p,i) * P(q,i);
+		        else L(pq,ij) = P(p,i) * P(q,j) + P(q,i) * P(p,j);
+	    }
+      }
+
+  Tsr.mat = L * tsr.mat * t(L);
+
+  return Tsr;
+}
+
+
+/*!
+  \brief Computes the inversion for a symmetric \f$ 4^{th} \f$ order tensor.
+
+  Computes \f$ T^{-1} \f$ such that \f$ T :: T^{-1} = Id \f$
+
+  \return \f$ T^{-1} \f$ 
+*/
+
+//----------------------------------------------------
+template <class T> symtensor4<T> symtensor4<T>::inv ()
+//----------------------------------------------------
+{
+  assert (dim1());
+
+  matrix<T> true_mat = (matrix<T>) (*this);
+  matrix<T> true_mat_inv = gauss (true_mat);
+
+  return symtensor4<T> (true_mat_inv);
+}
+
+
+/*!
+  \brief Computes the p-norm for symmetric \f$ 4^{th} \f$ order tensor.
+
+  Let \f$ T \f$ be a \f$ n \times n \times m \times m \f$ symmetric tensor. Let \f$ A \f$ be its associated matrix. Then, the tensor p-norm is given by: \n
+  \f$ \left\| A \right\|_p = \displaystyle \left( \sum_{i=1}^{n} \sum_{j=1}^{m} \left| a_{ij} \right|^p \right)^{1/p} \f$.\n
+  The special case p = 2 is the Frobenius norm also called the Hilbert-Schmidt norm.
+
+  \param p value for p-norm
+  \return \f$ \left\| T \right\|_p \f$
+
+*/
+
+//------------------------------------------------
+template <class T> T symtensor4<T>::p_norm (int p)
+//------------------------------------------------
+{
+  assert (p > 0);
+
+  T sum = 0;
+  for (int i=1; i<=dim1(); i++)
+    { for (int j=1; j<=dim3(); j++)
+	sum += pow (mat(i,j), p);
+      for (int j=dim3()+1; j<=mat.Columns(); j++)
+	sum += pow (sqrt(2) * mat(i,j), p);
+    }
+
+  for (int i=dim1()+1; i<=mat.Rows(); i++)
+    { for (int j=1; j<=dim3(); j++)
+	sum += pow (sqrt(2) * mat(i,j), p);
+      for (int j=dim3()+1; j<=mat.Columns(); j++)
+	sum += pow (2 * mat(i,j), p);
+    }
+
+  return pow (sum, (long double) 1/p);
+}
+
+
 //-----------------------------------------------------------------------------
 template <class Tf> ostream& operator << (ostream& s, const symtensor4<Tf>& st)
 //-----------------------------------------------------------------------------
