@@ -2,12 +2,16 @@
  *                 Mathematical Object Library                 *
  *      class tensor4 : declarations for 4th order tensors     *
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 2.4.0                   *
+ *                   GNU/linux version 2.6.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 2004,2005,2006,2007,2008,2009,2010 COLLARD Christophe
+ * copyright © 2004,2005,2006,2007,2008,2009,2010 Centre National de la Recherche Scientifique
+ * copyright © 2004,2005,2006,2007,2008,2009,2010 Arts et Métiers ParisTech
+ * copyright © 2004,2005,2006,2007 Université de Valenciennes et du Hainaut Cambrésis
  * copyright © 2004,2005,2006,2007,2008,2009,2010 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
  * copyright © 2004,2005,2006,2007 Laboratoire de Mathématiques et ses Applications de Valenciennes (LAMAV)
+ * copyright © 2010 Centre d'Elaboration de Matériaux et d'Etudes Structurales (CEMES - CNRS)
  ***************************************************************/
 
 /*! \namespace mol
@@ -41,10 +45,14 @@
     \endhtmlonly
 
     \author copyright \htmlonly &#169; \endhtmlonly 2004, 2005, 2006, 2007, 2008, 2009, 2010 Christophe COLLARD \n
+            copyright \htmlonly &#169; 2004, 2005, 2006, 2007, 2008, 2009, 2010 Centre National de la Recherche Scientifique \endhtmlonly \n
+	    copyright \htmlonly &#169; 2004, 2005, 2006, 2007, 2008, 2009, 2010 Arts et M&#233;tiers ParisTech \endhtmlonly \n
+	    copyright \htmlonly &#169; 2004, 2005, 2006, 2007 Universit&#233; de Valenciennes et du Hainaut Cambr&#233;sis \endhtmlonly \n
 	    copyright \htmlonly &#169; 2004, 2005, 2006, 2007, 2008, 2009, 2010 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	    copyright \htmlonly &#169; 2004, 2005, 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-    \version 2.4.0
-    \date 2004-2010
+	    copyright \htmlonly &#169; 2010 Centre d'Elaboration de Mat&#233;riaux et d'Etudes Structurales (CEMES - CNRS) \endhtmlonly \n
+    \version 2.6.0
+    \date 2004-2011
     \bug none
     \warning none
 */
@@ -151,13 +159,18 @@ template <class T> class tensor4
 
     template <class Tf> friend tensor4<Tf> Id4s (int);  // returns identity tensor for symmetric tensors : Tijkl = Tjikl = Tijlk
     template <class Tf> friend tensor4<Tf> change_basis (const matrix<Tf>&, const tensor4<Tf>&);
+    tensor4<T> inv () const {return matrix2symtensor (gauss (symtensor2matrix(*this)));}  // computes inversion operator for symmetrix tensors defined as 4th order tensors
 
     template <class Tf> friend ostream& operator << (ostream&, const tensor4<Tf>&);  // overloads output stream for tensors
     template <class Tf> friend istream& operator >> (istream&, const tensor4<Tf>&);  // overloads output stream for tensors
+
+    // i/o disk
     virtual void save (const char*);   // writes to disk
     virtual void save (const string&); // writes to disk
     virtual void read (const char*);   // reads from disk
     virtual void read (const string&);   // reads from disk
+
+    T p_norm (int p) {return tensor2matrix (*this).p_norm(p);}      // computes p-norm for matrix
     tensor4<T>& approximation ();
 };
 
