@@ -1,12 +1,16 @@
 /***************************************************************
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 2.4.0                   *
+ *	             GNU/linux version 2.6.0                   *
  *            software under General Public License            *
  ***************************************************************
- * copyright © 2003,2004,2005,2006,2007,2008,2009 COLLARD Christophe
+ * copyright © 2003,2004,2005,2006,2007,2008,2009,2010 COLLARD Christophe
  * copyright © 2003,2004 CREUSE Emmanuel
- * copyright © 2003,2004,2005,2006,2007,2008,2009 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
+ * copyright © 2003,2004,2005,2006,2007,2008,2009,2010 Centre National de la Recherche Scientifique
+ * copyright © 2003,2004,2005,2006,2007,2008,2009,2010 Arts et Métiers ParisTech
+ * copyright © 2003,2004,2005,2006,2007 Université de Valenciennes et du Hainaut-Cambrésis
+ * copyright © 2003,2004,2005,2006,2007,2008,2009,2010 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
  * copyright © 2003,2004,2005,2006,2007 Laboratoire de Mathématiques et ses Applications de Valenciennes (LAMAV)
+ * copyright © 2010 Centre d'Elaboration de Matériaux et d'Etudes Structurales (CEMES - CNRS)
  ***************************************************************/
 
 /*
@@ -453,6 +457,13 @@ int test_symmatrix (int detail)
   if (detail) affiche("diagonal", diagonal(matww)==diag);
   else result *= (diagonal(matww)==diag);
 
+  symmatrix<double> nca(9);
+  for (int i=1; i<=9; i++)
+    for (int j=i; j<=9; j++)
+      nca(i,j) = i*i+2*j;
+  if (detail) affiche ("p-norm (with p=2)", abs (sqrt (tr (t(nca) * nca)) - nca.p_norm(2)) < epsilon);
+  else result *= (abs (sqrt (tr (t(nca) * nca)) - nca.p_norm(2)) < epsilon);
+
   symmatrix<long double> Ids(5);
   for (int i=1; i<=5; i++)
     Ids (i,i) = 1;
