@@ -1,10 +1,13 @@
 /***************************************************************
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 2.4.0                   *
+ *	             GNU/linux version 2.6.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 2009,2010 COLLARD Christophe
+ * copyright © 2009,2010 Centre National de la Recherche Scientifique
+ * copyright © 2009,2010 Arts et Métiers ParisTech
  * copyright © 2009,2010 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
+ * copyright © 2010 Centre d'Elaboration de Matériaux et d'Etudes Structurales (CEMES - CNRS)
  ***************************************************************/
 
 /*
@@ -74,6 +77,10 @@
 #include "MOL++/tensors4.h"
 #endif
 
+#if !defined(__CONVERSION_TOOLS_H)
+#include "MOL++/conversion tools.h"
+#endif
+
 #if !defined(__ABAQUS_CONVERSION_TOOLS_H)
 #include "MOL++/abaqus conversion tools.h"
 #endif
@@ -171,6 +178,34 @@ int test_symtensor4 (int detail)
   if (detail) affiche ("constructor with default value", st0==t0 && st00==t00);
   else result *= (st0==t0 && st00==t00);
 
+  tensor4<long double> tsrs = st1;
+  if (detail) affiche ("cast operator () symtensor4 -> tensor4", t1==tsrs);
+  else result *= (t1==tsrs);
+
+  symtensor4<long double> stsr (t1);
+  matrix<long double> stsr_mat = (matrix<long double>) stsr;
+  if (detail) affiche ("cast operator () symtensor4 -> matrix", symtensor2matrix(t1)==stsr_mat);
+  else result *= (symtensor2matrix(t1)==stsr_mat);
+
+  if (detail) affiche ("cast constructor with matrix", stsr == symtensor4<long double> (stsr_mat));
+  else result *= (stsr == symtensor4<long double> (stsr_mat));
+
+  symmatrix<long double> smat (6,6);
+  for (int i=1; i<=6; i++)
+    for (int j=i; j<=6; j++)
+      smat (i,j) = 2*i + 5*j;
+  if (detail) affiche ("cast constructor with symmetric matrix", matrix2symtensor (smat) == symtensor4<long double> (smat));
+  else result *= (matrix2symtensor (smat) == symtensor4<long double> (smat));
+
+  symtensor4<long double> t1_copy = t1;
+  if (detail) affiche ("copy constructor", t1 == t1_copy);
+  else result *= (t1 == t1_copy);
+
+  symtensor4<long double> t1_temp = &(t1+t1-t1);
+  if (detail) affiche ("constructor with temporary objects", t1 == t1_temp);
+  else result *= (t1 == t1_temp);
+
+
   /*
   tensor3<long double> tens3 (5,7,2);
   for (int j=1; j<=5; j++)
@@ -454,13 +489,17 @@ int test_symtensor4 (int detail)
   if (detail) affiche("Id4sym", Id_4==Id_4s);
   else result *= (Id_4==Id_4s);
 
-  /*
   matrix<long double> P(3,3);
   matrix<long double> passlg(3,3); // orthogonal matrix
-  tensor4<long double> tsr(3), tsr2(3), C(3);
+  tensor4<long double> C(3);
   for (int i=1; i<=3; i++)
     for (int j=1; j<=3; j++)
       P(i,j) = (2.52*i - 1.25*j + 25 * (i==j)) * 1E-2;
+  for (int i=1; i<=3; i++)
+    for (int j=1; j<=3; j++)
+      for (int k=1; k<=3; k++)
+	for (int l=1; l<=3; l++)
+	  C(i,j,k,l) = 2 * (i + j) / (k * l);
   long double phi1=173.464, phi=104.504, phi2=34.5;
   passlg(1,1) =  cos(phi1) * cos(phi2)  -  sin(phi1) * cos(phi) * sin(phi2);
   passlg(1,2) = -cos(phi1) * sin(phi2)  -  sin(phi1) * cos(phi) * cos(phi2);
@@ -471,26 +510,9 @@ int test_symtensor4 (int detail)
   passlg(3,1) =  sin(phi)  * sin(phi2);
   passlg(3,2) =  sin(phi)  * cos(phi2);
   passlg(3,3) =  cos(phi);
-  for (int i=1; i<=3; i++)
-    for (int j=1; j<=3; j++)
-      for (int k=1; k<=3; k++)
-	for (int l=1; l<=3; l++)
-	  { C(i,j,k,l) = (2.4*i -j / (1.2*i) ) * k - 3.1*l;
-	    for (int p=1; p<=3; p++)
-	      for (int q=1; q<=3; q++)
-		for (int m=1; m<=3; m++)
-		  for (int n=1; n<=3; n++)
-		    { tsr(p,q,m,n) += P(p,i) * P(q,j) * P(m,k) * P(n,l) * C(i,j,k,l);
-		      tsr2(p,q,m,n) += passlg(p,i) * passlg(q,j) * passlg(m,k) * passlg(n,l) * C(i,j,k,l);
-		    }
-	  }
-  long double scalar = .1;
-  test = true;
-  test *= (tsr2 == change_basis(passlg,C));
-  test *= (tsr == change_basis(P,C));
-  if (detail) affiche ("change basis", test);
-  else result *= test;
-  */
+  symtensor4<long double> Cs = C;
+  if (detail) affiche ("change basis", change_basis (passlg, C) == change_basis (passlg, Cs)  &&  change_basis (P, C) == change_basis (P, Cs));
+  else result *= (change_basis (passlg, C) == change_basis (passlg, Cs)  &&  change_basis (P, C) == change_basis (P, Cs));
 
   symtensor4<long double> stint (7,7,5,5);
   for (int i=1; i<=7; i++)
@@ -509,18 +531,17 @@ int test_symtensor4 (int detail)
     for (int j=1; j<=6; j++)
       if (i==j) mat(i,j) = 20*i + 30*j;
       else mat(i,j) = 2*i + 3*j;
-  symtensor4<long double> Tsquare (mat);
-  symtensor4<long double> Tsquare_inv (mat.inv());
+  symtensor4<long double> Tsquare (abaqus::matrix2symtensor (mat));
+  tensor4<long double> T4square = abaqus::matrix2symtensor (mat);
+  symtensor4<long double> Tsquare_inv = T4square.inv();
   if (detail) affiche("inv", Tsquare.inv()==Tsquare_inv);
   else result *= (Tsquare.inv()==Tsquare_inv);
 
   if (detail) affiche("gauss", gauss(Tsquare)==Tsquare_inv);
   else result *= (gauss(Tsquare)==Tsquare_inv);
 
-  symtensor4<long double> Tinv = Tsquare;
-  inv(Tinv, epsilon);
-  //  if (detail) affiche("inv", inv(Tinv) && Tinv==Tsquare_inv);
-  //  else result *= (inv(Tinv) && Tinv==Tsquare_inv);
+  if (detail) affiche ("p-norm (with p=2)", abs (stsr.p_norm (2) - stsr_mat.p_norm(2)) < epsilon * stsr.p_norm (2));
+  else result *= (abs (stsr.p_norm (2) - stsr_mat.p_norm(2)) < epsilon * stsr.p_norm (2));
 
   string filename = "symtenseur4.res";
   stint.save(filename);
