@@ -1,11 +1,15 @@
 /***************************************************************
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 2.4.0                   *
+ *	             GNU/linux version 2.6.0                   *
  *            software under General Public License            *
  ***************************************************************
- * copyright © 2003,2004,2005,2006,2007,2008,2009 COLLARD Christophe
- * copyright © 2003,2004,2005,2006,2007,2008 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
+ * copyright © 2003,2004,2005,2006,2007,2008,2009,2010 COLLARD Christophe
+ * copyright © 2003,2004,2005,2006,2007,2008,2009,2010 Centre National de la Recherche Scientifique
+ * copyright © 2003,2004,2005,2006,2007,2008,2009,2010 Arts et Métiers ParisTech
+ * copyright © 2003,2004,2005,2006,2007 Université de Valenciennes et du Hainaut-Cambrésis
+ * copyright © 2003,2004,2005,2006,2007,2008,2009,2010 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
  * copyright © 2003,2004,2005,2006,2007 Laboratoire de Mathématiques et ses Applications de Valenciennes (LAMAV)
+ * copyright © 2010 Centre d'Elaboration de Matériaux et d'Etudes Structurales (CEMES - CNRS)
  ***************************************************************/
 
 /*
@@ -75,12 +79,17 @@
 #include "MOL++/tensors4.h"
 #endif
 
+#if !defined(__CONVERSION_TOOLS_H)
+#include "MOL++/conversion tools.h"
+#endif
+
 #if !defined(__AFFICHE_h)
 #include "tests/affiche.h"
 #endif
 
 using namespace mol;
 
+
 //===========================
 int test_tensor4 (int detail)
 //===========================
@@ -428,6 +437,15 @@ int test_tensor4 (int detail)
   if (detail) affiche ("change basis", test);
   else result *= test;
 
+  matrix<long double> mat (6,6);
+  for (int i=1; i<=6; i++)
+    for (int j=1; j<=6; j++)
+      if (i==j) mat(i,j) = 20*i + 30*j;
+      else mat(i,j) = 2*i + 3*j;
+  tensor4<long double> Tsquare (matrix2symtensor(mat));
+  if (detail) affiche ("inv", (Tsquare || Tsquare.inv()) == Id4s<long double>(3));
+  else result *= ((Tsquare || Tsquare.inv()) == Id4s<long double>(3));
+
   t6.save("tensor4.res");
   tensor4<long double> ncd(t6.dim1(), t6.dim2(), t6.dim3(), t6.dim4());
   ncd.read("tensor4.res");
@@ -441,6 +459,16 @@ int test_tensor4 (int detail)
   if (detail) affiche("read/write with a string argument", ncd2==t6);
   else result *= (ncd2==t6);
 
+  matrix<long double> Tmat (36,9);
+  for (int i=1; i<=36; i++)
+    for (int j=1; j<=9; j++)
+      Tmat (i,j) = pow(i,3) - 7*j;
+  tensor4<long double> T = matrix2tensor (Tmat, 6, 6, 3, 3);
+  test = abs (T.p_norm (2) - Tmat.p_norm(2)) < epsilon * Tmat.p_norm (2);
+  test *= abs (Tsquare.p_norm (2) - mat.p_norm(2)) < epsilon * mat.p_norm (2);
+  if (detail) affiche ("p-norm (with p=2)", test);
+  else result *= test;
+
   cout << endl;  
 
   cout << "============================================================== \n";
